// Add git commit author and date to metadata
var git = require('nodegit');
var path = require('path');

var pathToRepo = path.resolve("content");
var getCommits = function(file, cb) {
  git.Repository.open(pathToRepo)
    .then(function(r) {
      repo = r;
      return repo.getMasterCommit();
    })
    .then(function(firstCommitOnMaster){
      walker = repo.createRevWalk();
      walker.push(firstCommitOnMaster.sha());
      walker.sorting(git.Revwalk.SORT.Time);

      return walker.fileHistoryWalk(file, 500);
    })
    .then(function(resultingArrayOfCommits) {
      if(resultingArrayOfCommits.length == 0) {
        cb(undefined);
        return;
      }
      var commits = [];
      resultingArrayOfCommits.forEach(function(e) {
        commits.push(e.commit);
      });
      cb(commits);
    })
    .done();
}

module.exports = function() {
  var tr = this;
  tr.fancyname = "transformer git";
  
  tr.pre = function(cb, articleList) {
    var todo = articleList.length;
    articleList.forEach(function(article) {
      var file = path.relative(pathToRepo, article.origFile);
      getCommits(file, function(commits) {
        if(commits !== undefined) {
          var newestCommit = commits[0];
          var oldestCommit = commits[commits.length-1];
          
          if(newestCommit != newestCommit) {
            console.log("Newest commit:");
            printCommit(newestCommit);
          }
          article.metadata.date = {
            creation: oldestCommit.date(),
            modification: newestCommit.date(),
            modified: newestCommit !== oldestCommit
          }
          article.metadata.author = {
            name: oldestCommit.author().name(),
            email: oldestCommit.author().email()
          }
          article.commits = commits;
        }
        
        todo--;
        if(todo == 0) cb();
      });
    });
  }
  
  tr.forEachPage = function(article, cb) {
    cb();
  }
}

